using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;


namespace CTCShell
{
    public unsafe class Ctccomm32v2
    {   
    //
    //  MODULE:   ctccom32v2.bas
    //
    //  PURPOSE:
    //    Contains declarations for Ctccom32v2 DLL.
    //


    // constants for binary protocol
    public const int CT_ASCII   = 10;
    public const int CT_REQUEST   = 20;
    public const int CT_REPLY   = 21;
    public const int CT_ACK   = 100;
    public const int CT_NAK   = 101;
    public const int CT_RANGE   = 102;
    public const int CT_VALUE   = 104;

    // function return values
    public const int FAILURE = 0;
    public const int SUCCESS  = 1;
    public const int ERROR_UNKNOWN  = 255;
    public const int ERROR_WIN_TIMER  = 246;
    public const int ERROR_NET_TIMEOUT   = 236;
    public const int ERROR_NET_NOTREADY   = 235;
    public const int ERROR_CT_RANGE   = 216;
    public const int ERROR_BAD_CHECKSUM   = 215;
    public const int ERROR_BAD_LENGTH   = 214;
    public const int ERROR_BAD_RESPONSE   = 206;
    public const int ERROR_NOT_CT_PROTOCOL   = 205;
    public const int ERROR_UNSUPP_REMOTE_OPER   = 196;
    public const int ERROR_UNKNOWN_REM_NODE   = 195;
    public const int ERROR_REMOTE_ACCESS   = 194;
    public const int ERROR_OBJECT_FILE_NOT_FOUND   = 186;
    public const int ERROR_FILE_WRONG_ARCH   = 185;
    public const int ERROR_FILE_NOT_OBJECT   = 184;
    public const int ERROR_BAD_OBJECT_RECORD   = 183;
    public const int ERROR_BAD_OBJECT_LENGTH   = 182;
    public const int ERROR_BAD_POSITION   = 181;
    public const int ERROR_EOF   = 180;
    public const int ERROR_BAD_PORT   = 179;
    public const int ERROR_NET_IN_USE   = 178;
    public const int ERROR_SERIAL_TIMEOUT   = 170;
    public const int ERROR_WARNING   = 169;
    public const int ERROR_ILLEGAL_BAUD_RATE   = 168;
    public const int ERROR_ILLEGAL_PARITY_SETTING   = 167;
    public const int ERROR_ILLEGAL_WORD_LENGTH   = 166;
    public const int ERROR_ILLEGAL_STOP_BITS   = 16;
    public const int ERROR_ERROR   = 160;
    public const int ERROR_PORT_NOT_FOUND   = 159;
    public const int ERROR_PORT_IN_USE   = 158;

    // Constants for port Structures
    public const int NO_PORT   = 0;
    public const int SERIAL   = 1;
    public const int CTCNET   = 2;
    public const int CTCUDP   = 3;
    public const int CTCTCP   = 4;

    // Constants for Parity
    public const byte NO_PARITY = 0;
    public const byte ODD_PARITY = 1;
    public const byte EVEN_PARITY = 2;
    public const byte MARK_PARITY = 3;
    public const byte SPACE_PARITY = 4;

    // Constants for Data Bits
    public const byte FOUR_DATA_BITS = 4;
    public const byte FIVE_DATA_BITS = 5;
    public const byte SIX_DATA_BITS = 6;
    public const byte SEVEN_DATA_BITS = 7;
    public const byte EIGHT_DATA_BITS = 8;

    // Constants for Stop Bits
    public const byte ONE_STOP_BITS = 0;
    public const byte ONE_FIVE_STOP_BITS = 1;
    public const byte TWO_STOP_BITS = 2;

/*
        // program file statistics
    struct CT_PROG_COUNTS
    {
        int lSteps;  
        int lBytes;   
        int lErrors;   
        int lWarnings;  
    }

    // data table row structure
    public const int CT_MAX_COLS = 255;
    struct CT_DT_ROW
    {
        int lRow; 
        int *lColumns;  
    }

    // io count structure
    struct CT_IO_COUNT
    {
        int lFlags;  
        int lDigitalInputs;  
        int lDigitalOutputs;  
        int lStepperMotors;  
        int lServoMotors;  
        int lAnalogInputs;  
        int lAnalogOutputs;  
    }

    // misc io count structure
    struct CT_MISC_IO_COUNT
    {
        int lProtoStructures;  
        int lHighSpeedCounters;  
        int lThumbwheels;  
        int lDisplays;  
    }

    // step status bank structure
    struct CT_TASK_BANK
    {    
        int lMode;  
        int lFaultCode;  
        int lFaultStep;  
        int lFaultData;  
        int *lStep;  
        int *lTaskMask1;  
        int *lTaskMask2;  
    }
    
    // connection information structure
    struct CT_CONN_INFO
    {
        int lConnStructure;  
        int lSrcAddr;  
        int lDestAddr;  
        int lTimeout;  
        bool fConnected;
        byte *srcEAddr;
        byte *destEAddr;
    }

    // message information structure
    struct CT_MSG_INFO
    {
        int iMsglen;  
        string *sMsgbuf; // * 256
        int *iRsplen;  
        string *sRspbuf; // * 256
    }

    // error information structure
    struct CT_ERR_INFO
    {
        int lCode;  
        string *sMessage; // * 256
    }
*/    
    // network access functions 
    [DllImport("Ctccom32v2.dll")] public static extern int CtOpenConnection 
        (int ConnectID, int CommPort, int Address);
    [DllImport("Ctccom32v2.dll")] public static extern int CtInitConnection
        (int ConnectID, int BaudRate, byte Parity, byte DataBits, byte StopBits);
    [DllImport("Ctccom32v2.dll")] public static extern int CtCloseConnection 
        (int ConnectID);
    [DllImport("Ctccom32v2.dll")] public static extern int CtSetConnectionTimeout
        (int ConnectID, int MilliSeconds);

            // windows functions that may be used
    [DllImport("wsock32.dll")] public static extern int inet_addr
        (string ipAddress);      

    // register functions
    [DllImport("Ctccom32v2.dll")] public static extern int CtGetRegister 
        (int ConnectID, int RegAddress, int *RegValue);
    [DllImport("Ctccom32v2.dll")] public static extern int CtPutRegister
        (int ConnectID, int RegAddress, int RegValue);

        /*  
       Declare Auto Function CtGetRegister16 Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal RegAddress  , ByRef Reg16Val  )  
       Declare Auto Function CtGetRegister50 Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal RegBank  , ByRef Reg50Val  )  
       Declare Auto Function CtPutRegister Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal RegAddress  , ByVal RegValue  )  

       // flag functions
       Declare Auto Function CtGetFlag Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal FlagAddress  , ByRef FlagValue  )  
       Declare Auto Function CtPutFlag Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal FlagAddress  , ByVal FlagValue  )  

       // digital io functions
       Declare Auto Function CtGetDigitalInput8 Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal Bank  , ByRef Value  )  
       Declare Auto Function CtGetDigitalOutput8 Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal Bank  , ByRef Value  )  
       Declare Auto Function CtGetDigitalInput128 Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal Bank  , ByRef Dig128Val  )  
       Declare Auto Function CtGetDigitalOutput128 Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal Bank  , ByRef Dig128Val  )  

       // analog io functions
       Declare Auto Function CtGetAnalogInput Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal Index  , ByRef Value  )  
       Declare Auto Function CtGetAnalogOutput Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal Index  , ByRef Value  )  
       Declare Auto Function CtPutAnalogOutput Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal Index  , ByVal Value  )  
       Declare Auto Function CtGetAnalogInput32 Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal Bank  , ByRef Analog32Val  )  
       Declare Auto Function CtGetAnalogOutput32 Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal Bank  , ByRef Analog32Val  )  

       // servo functions
       Declare Auto Function CtGetServoPosition Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal Address  , ByRef Value  )  
       Declare Auto Function CtGetServoError Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal Address  , ByRef Value  )  
       Declare Auto Function CtGetServoInputs Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal Address  , ByRef Value  )  

       // data table functions
       Declare Auto Function CtGetDataTableDim Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByRef NumRows  , ByRef numCols  )  
       Declare Auto Function CtPutDataTableDim Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal NumRows  , ByVal numCols  )  
       Declare Auto Function CtGetDataTableLoc Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal Row  , ByVal Col  , ByRef Value  ) _
         
       Declare Auto Function CtPutDataTableLoc Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal Row  , ByVal Col  , ByVal Value  ) _
         
       Declare Auto Function CtGetDataTableRow Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal Row  , ByVal Col  , ByVal numCols  , _
           ByRef DtRow As CT_DT_ROW)  
       Declare Auto Function CtPutDataTableRow Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal Row  , ByVal Col  , ByVal numCols  , _
           ByRef DtRow As CT_DT_ROW)  

       // status functions
       Declare Auto Function CtGetStatus Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByRef CtStatus  )  
       Declare Auto Function CtPutStatus Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal CtStatus  )  
       Declare Auto Function CtGetTaskBank Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal Bank  , ByRef TaskStatus As CT_TASK_BANK)  

       // configuration functions
       Declare Auto Function CtGetModel Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByRef CtModel  , ByRef IsEA As Boolean)  
       Declare Auto Function CtGetConfig Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByRef CtConfig  )  
       Declare Auto Function CtPutConfig Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal CtConfig  )  
       Declare Auto Function CtGetIoCount Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByRef IoCounts As CT_IO_COUNT)  
       Declare Auto Function CtGetMiscIoCount Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByRef miscCounts As CT_MISC_IO_COUNT)  

       // program functions
       Declare Auto Function CtDownload Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal Filename As String, _
           ByRef ProgramCounts As CT_PROG_COUNTS)  
       Declare Auto Function CtUpload Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal Filename As String, _
           ByRef ProgramCounts As CT_PROG_COUNTS)  

       // miscellaneous functions
       Declare Auto Function CtGetConnectionInfo Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByRef ConnInfo As CT_CONN_INFO)  
       Declare Auto Function CtGetMessageInfo Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByRef MsgInfo As CT_MSG_INFO)  
       Declare Auto Function CtGetErrorInfo Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByRef ErrInfo As CT_ERR_INFO)  
       Declare Auto Function CtSendData Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal PktStructure  , ByVal Pkt As String, _
           ByVal PktLen  , ByVal ReturnPkt As String)  
       Declare Auto Function CtSendAscii Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByVal Pkt As String, ByVal PktLen  , _
           ByVal ReturnPkt As String, ByVal termChar As Byte)  

       // devicenet / distributed module functions
       Declare Auto Function CtLoadRemoteIOConfig Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByRef ProgramCounts As CT_PROG_COUNTS)  
       Declare Auto Function CtUnloadRemoteIOConfig Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByRef ProgramCounts As CT_PROG_COUNTS)  
       Declare Auto Function CtGetRemoteIOData Lib "Ctccom32v2.dll" _
           (ByVal ConnectID  , ByRef ProgramCounts As CT_PROG_COUNTS)  

   
   */
}
}
